#!/bin/bash
#------------------------------------------------------------------------------
# /var/install/bin/inadyn.sh
#
# Last Update: 2021-08-13 se, sebastian(dot)ertz(at)tk(minus)ertz(dot)de
#
# Copyright (c) 2001-2023 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------
[ -f /etc/config.d/inadyn ] && . /etc/config.d/inadyn


#------------------------------------------------------------------------------
# functions
#------------------------------------------------------------------------------
Log(){
	[[ "${INADYN_LOG_LEVEL:-notice}" == "info" || "${INADYN_LOG_LEVEL:-notice}" == "debug" ]] && logger -i -t "inadyn" "$*"
}
CacheLoad(){
	local ipaddr=""
	[[ $# -eq 1 && $1 && -s "/var/cache/inadyn/inadyn.$1.cache" ]] && ipaddr="$(<"/var/cache/inadyn/inadyn.$1.cache")"
	printf "%s" "$ipaddr"
}
CacheWrite(){
	if [[ $# -eq 2 && $1 && $2 ]]; then
		[[ -d /var/cache/inadyn/ ]] || mkdir -p /var/cache/inadyn/
		printf "%s" "$2" > "/var/cache/inadyn/inadyn.$1.cache"
		chmod 0600 "/var/cache/inadyn/inadyn.$1.cache"
		chown root:root "/var/cache/inadyn/inadyn.$1.cache"
	fi
}
InadynGetWanIp(){
	local cmd_curl
	local ipaddr=""
	if [[ ! -s /var/cache/inadyn/inadyn.wanip.cache || "$(stat --printf "%Y" "/var/cache/inadyn/inadyn.wanip.cache")" -lt $(($(printf "%(%s)T" -1)-(INADYN_CHECKIP_INTERVAL*60)+5)) ]]; then
		cmd_curl="$(command -v curl 2>/dev/null)" || exit 1
		[ "${INADYN_CHECKIP_FRITZBOX:-no}" = "yes" ] || exit 2
		ipaddr="$($cmd_curl -s \
			-H 'Content-Type: text/xml; charset="utf-8"' \
			-H 'SOAPAction: urn:schemas-upnp-org:service:WANIPConnection:1#GetExternalIPAddress' \
			-d '<?xml version="1.0" encoding="utf-8"?><s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"><s:Body><u:GetExternalIPAddress xmlns:u="urn:schemas-upnp-org:service:WANIPConnection:1" /></s:Body></s:Envelope>' \
			"http://${INADYN_CHECKIP_FRITZBOX_IP:-192.168.178.1}:49000/igdupnp/control/WANIPConn1" \
			|grep -Eo "\<[[:digit:]]{1,3}(\.[[:digit:]]{1,3}){3}\>")" || exit 3
		[[ $ipaddr ]] || exit 4
		Log "Check WAN-IP FRITZ!Box (${INADYN_CHECKIP_FRITZBOX_IP:-192.168.178.1}): $ipaddr"
		CacheWrite "wanip" "$ipaddr"
	else
		ipaddr="$(CacheLoad "wanip")"
		Log "Check WAN-IP FRITZ!Box (${INADYN_CHECKIP_FRITZBOX_IP:-192.168.178.1}): $ipaddr (cache)"
	fi
	[ "$ipaddr" ] || exit 2
	printf "%s" "$ipaddr"
}
InadynEventError(){
	local cmd_sendmail
	if [[ "${INADYN_MAIL:-no}" == "yes" && "$INADYN_MAIL_ADDR" ]]; then
		cmd_sendmail="$(command -v sendmail 2>/dev/null)" || exit 1
		Log "Error $INADYN_HOSTNAME $INADYN_ERROR_MESSAGE: send mail to $INADYN_MAIL_ADDR"
		{
			printf "From: Mailer-Daemon <root@%s>\n" "$(hostname -d)"
			printf "To: %s\n" "$INADYN_MAIL_ADDR"
			printf "Subject: In-a-Dyn Error %s %s\n" "$INADYN_HOSTNAME" "$INADYN_ERROR_MESSAGE"
			printf "\n"
			printf "Hostname: %s\n" "$INADYN_HOSTNAME"
			printf "ErrorCode: %s\n" "$INADYN_ERROR"
			printf "Message: %s\n" "$INADYN_ERROR_MESSAGE"
		} | "$cmd_sendmail" -t >/dev/null 2>&1
	fi
}
InadynEventUpdate(){
	local cmd_sendmail
	local ipaddr
	if [[ $INADYN_IP ]]; then
		ipaddr="$(CacheLoad "$INADYN_HOSTNAME")"
		if [ "$INADYN_IP" != "$ipaddr" ]; then
			CacheWrite "$INADYN_HOSTNAME" "$INADYN_IP"
			if [[ "${INADYN_MAIL:-no}" == "yes"  && $INADYN_MAIL_ADDR ]]; then
				cmd_sendmail="$(command -v sendmail 2>/dev/null)" || exit 1
				Log "$INADYN_HOSTNAME -> $INADYN_IP: send mail to $INADYN_MAIL_ADDR"
				{
					printf "From: Mailer-Daemon <root@%s>\n" "$(hostname -d)"
					printf "To: %s\n" "$INADYN_MAIL_ADDR"
					printf "Subject: In-a-Dyn %s -> %s\n" "$INADYN_HOSTNAME" "$INADYN_IP"
					printf "\n"
					printf "Hostname: %s\n" "$INADYN_HOSTNAME"
					printf "New IP: %s\n" "$INADYN_IP"
				} | "$cmd_sendmail" -t >/dev/null 2>&1
			fi
		fi
	fi
}


#==============================================================================
# Main
#==============================================================================
Main(){
	if [[ $# -eq 1 && $1 ]]; then
		rm -f /var/cache/inadyn/inadyn.wanip.cache
		INADYN_CHECKIP_FRITZBOX_IP="$1"
		INADYN_CHECKIP_INTERVAL="0"
		INADYN_PROVIDER="none"
		INADYN_USER="none"
	fi

	if [[ $INADYN_PROVIDER && $INADYN_USER ]]; then
		InadynGetWanIp
	elif [[ $INADYN_EVENT && $INADYN_HOSTNAME ]]; then
		case "$INADYN_EVENT" in
			error)  InadynEventError;;
			nochg)  ;;
			update) InadynEventUpdate;;
		esac
		printf "\n"
	else
		exit 1
	fi
}
Main "$@"

exit 0
